# MIDI SCRIPTER
# Copyright (c) 2005 Emanuel Borsboom.  All rights reserved.
#
# UnsafeProxy class, which allows you to call code that is running at a lower $SAFE level.
#

require 'thread'

class UnsafeProxy

	def initialize(obj, *methods)
                @obj = obj
		@methods = methods
		if (@methods.length == 0)
			@methods = obj.class.public_instance_methods(false)
			@methods.collect! { |x| x.intern }
		end

		thr = Thread.new do
			# Create instance variables at safe level 3 so they are tainted so that safe level 4 code can use them
			$SAFE = 3
                        @call_queue = Queue.new
                        @result_queue = Queue.new
		end
		thr.join
		self.taint # Taint self so that safe level 4 code can use me
		
		@thread = Thread.new do
                        #puts "proxy starting"
                        
                        while true
                                # "proxy waiting"
                                meth, args = @call_queue.pop
                                #puts "proxy meth=#{meth} args=#{args}"
                                if meth == '__DESTOY__'
                                        #puts "proxy exiting"
                                        Thread.exit
                                end
                                to_push = nil
                                begin
                                        if !@methods.include? meth
                                                raise SecurityError, "No permission to call method '" + @meth.to_s + "'", caller
                                        end
                                        result = @obj.send(meth, *args)
                                        #puts "proxy pushing result"
                                        to_push = ['result', result]
                                rescue Exception => excep
                                        #puts "proxy pushing exception #{excep}"
                                        to_push = ['exception', excep]
                                ensure
                                        if to_push == nil
                                                to_push = ['exception', Exception.new("This shouldn't ever happen")]
                                        end
                                        @result_queue.push to_push
                                end
                        end
                end
	end
        
        def destroy
                @call_queue.push ['__DESTOY__', []]
        end
	
	def method_missing(meth, *args)
                #puts "method_missing pushing #{meth} #{args}"
                @call_queue.push [meth, args]
                #puts "method_missing waiting"
                type, value = @result_queue.pop
                #puts "method_missing type=#{type}"
                if type == "exception"
                        #puts "method_missing raising exception"
                        raise value, value.to_s, caller[2..-1]
                else
                        #puts "method_missing returning value"
                        return value
                end
	end

end
